Class: Symbol
Instance Method Summary collapse
-
#const_lookup(ns = Object) ⇒ Object
looks up this symbol as a constant defined in 'ns' (Object by default).
Instance Method Details
#const_lookup(ns = Object) ⇒ Object
looks up this symbol as a constant defined in 'ns' (Object by default)
3 4 5 |
# File 'lib/rbkb/extends/symbol.rb', line 3 def const_lookup(ns = Object) to_s.const_lookup(ns) end |